"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Animators", function () {
    describe("EasingAnimator", function () {
        var animator;
        beforeEach(function () {
            animator = new Plottable.Animators.Easing();
        });
        describe("getters and setters", function () {
            it("can get and set the start delay", function () {
                var adjustedStartDelay = animator.startDelay() + 10;
                chai_1.assert.strictEqual(animator.startDelay(adjustedStartDelay), animator, "setter mode returns the Animator");
                chai_1.assert.strictEqual(animator.startDelay(), adjustedStartDelay, "retrieved the set value");
            });
            it("can get and set the step duration", function () {
                var adjustedStepDuration = animator.stepDuration() + 10;
                chai_1.assert.strictEqual(animator.stepDuration(adjustedStepDuration), animator, "setter mode returns the Animator");
                chai_1.assert.strictEqual(animator.stepDuration(), adjustedStepDuration, "retrieved the set value");
            });
            it("can get and set the step delay", function () {
                var adjustedStepDelay = animator.stepDelay() + 10;
                chai_1.assert.strictEqual(animator.stepDelay(adjustedStepDelay), animator, "setter mode returns the Animator");
                chai_1.assert.strictEqual(animator.stepDelay(), adjustedStepDelay, "retrieved the set value");
            });
            it("can get and set the max total duration", function () {
                var setMaxTotalDuration = 10;
                chai_1.assert.strictEqual(animator.maxTotalDuration(setMaxTotalDuration), animator, "setter mode returns the Animator");
                chai_1.assert.strictEqual(animator.maxTotalDuration(), setMaxTotalDuration, "retrieved the set value");
            });
            it("can get and set the easing mode", function () {
                chai_1.assert.strictEqual(animator.easingMode("bounce"), animator, "setter mode returns the Animator");
                chai_1.assert.strictEqual(animator.easingMode(), "bounce", "retrieved the set value");
            });
        });
        describe("total time", function () {
            var NUM_STEPS = 10;
            it("includes start delay in the total time", function () {
                var startDelay = 123;
                animator.startDelay(startDelay);
                animator.stepDelay(0);
                animator.stepDuration(0);
                var expectedTotalTime = startDelay;
                chai_1.assert.strictEqual(animator.totalTime(NUM_STEPS), expectedTotalTime, "total time includes start delay (when other values are 0)");
            });
            it("includes step delay in the total time", function () {
                animator.startDelay(0);
                var stepDelay = 123;
                animator.stepDelay(stepDelay);
                animator.stepDuration(0);
                var expectedTotalTime = (NUM_STEPS - 1) * stepDelay;
                chai_1.assert.strictEqual(animator.totalTime(NUM_STEPS), expectedTotalTime, "total time increased by each delay between steps");
            });
            it("includes step duration in the total time", function () {
                animator.startDelay(0);
                animator.stepDelay(0);
                var stepDuration = 123;
                animator.stepDuration(stepDuration);
                var expectedTotalTime = stepDuration;
                chai_1.assert.strictEqual(animator.totalTime(NUM_STEPS), expectedTotalTime, "total time increased by one step duration");
            });
        });
        describe("max total duration", function () {
            var EPSILON = 0.0000001; // floating-point errors
            var NUM_STEPS = 10;
            var ORIGINAL_STEP_DELAY = 100;
            var ORIGINAL_STEP_DURATION = 100;
            var totalTimeLimit;
            beforeEach(function () {
                animator.stepDelay(ORIGINAL_STEP_DELAY);
                animator.stepDuration(ORIGINAL_STEP_DURATION);
                totalTimeLimit = animator.totalTime(NUM_STEPS);
                animator.maxTotalDuration(totalTimeLimit);
            });
            it("restricts the total time when the number of steps increases", function () {
                var moreStepsTotalTime = animator.totalTime(2 * NUM_STEPS);
                chai_1.assert.closeTo(moreStepsTotalTime, totalTimeLimit, EPSILON, "adding more steps does not increase the total time past the limit");
            });
            it("restricts the total time when the step delay increases", function () {
                animator.stepDelay(2 * ORIGINAL_STEP_DELAY);
                var longerStepDelayTotalTime = animator.totalTime(NUM_STEPS);
                chai_1.assert.closeTo(longerStepDelayTotalTime, totalTimeLimit, EPSILON, "increasing step delay does not increase the total time past the limit");
            });
            it("restricts the total time when the step duration increases", function () {
                animator.stepDuration(2 * ORIGINAL_STEP_DURATION);
                var longerStepDurationTotalTime = animator.totalTime(NUM_STEPS);
                chai_1.assert.closeTo(longerStepDurationTotalTime, totalTimeLimit, EPSILON, "increasing step duration does not increase the total time past the limit");
            });
            it("allows the start delay to increase the total time", function () {
                var originalTotalTime = animator.totalTime(NUM_STEPS);
                animator.startDelay(100);
                var totalTimeWithStartDelay = animator.totalTime(NUM_STEPS);
                var expectedTotalTime = animator.startDelay() + originalTotalTime;
                chai_1.assert.closeTo(totalTimeWithStartDelay, expectedTotalTime, EPSILON, "start delay is added on to the max total duration");
            });
        });
    });
});
