"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Animators", function () {
    describe("Null Animator", function () {
        it("has a total animation time of zero", function () {
            var animator = new Plottable.Animators.Null();
            chai_1.assert.strictEqual(animator.totalTime(null), 0);
        });
        it("applies the requested attributes to the selection", function () {
            var attributeName = "foo";
            var attributeValues = ["A", "B", "C"];
            var attributeProjector = function (d, i) { return attributeValues[i]; };
            var attrToAppliedProjector = {};
            attrToAppliedProjector[attributeName] = attributeProjector;
            var div = TestMethods.generateDiv();
            var elementName = "g";
            attributeValues.forEach(function () { return div.append(elementName); });
            var elements = div.selectAll(elementName);
            var animator = new Plottable.Animators.Null();
            animator.animate(elements, attrToAppliedProjector);
            elements.each(function (d, i) {
                var actualAttribute = d3.select(this).attr(attributeName);
                var expectedAttribute = attributeProjector(d, i);
                chai_1.assert.strictEqual(actualAttribute, expectedAttribute, "set the correct attribute on element with index " + i);
            });
            div.remove();
        });
    });
});
