"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Gridlines", function () {
    var div;
    var xScale;
    var yScale;
    var gridlines;
    beforeEach(function () {
        div = TestMethods.generateDiv();
        xScale = new Plottable.Scales.Linear();
        xScale.domain([0, 10]);
        yScale = new Plottable.Scales.Linear();
        yScale.domain([0, 10]);
        gridlines = new Plottable.Components.Gridlines(xScale, yScale);
    });
    it("sets ranges of scales to the Gridlines dimensions when layout is computed", function () {
        gridlines.renderTo(div);
        chai_1.assert.deepEqual(xScale.range(), [0, Plottable.Utils.DOM.elementWidth(div)], "x scale range extends to the width of the div");
        chai_1.assert.deepEqual(yScale.range(), [Plottable.Utils.DOM.elementHeight(div), 0], "y scale range extends to the height of the div");
        div.remove();
    });
    it("draws gridlines on ticks of its scales and updates when scale update", function () {
        gridlines.renderTo(div);
        var xGridlines = gridlines.content().select(".x-gridlines").selectAll("line");
        var xTicks = xScale.ticks();
        chai_1.assert.strictEqual(xGridlines.size(), xTicks.length, "There is an x gridline for each x tick");
        xGridlines.each(function (gridline, i) {
            var x = TestMethods.numAttr(d3.select(this), "x1");
            chai_1.assert.closeTo(x, xScale.scale(xTicks[i]), window.Pixel_CloseTo_Requirement, "x gridline drawn on ticks");
        });
        var yGridlines = gridlines.content().select(".y-gridlines").selectAll("line");
        var yTicks = yScale.ticks();
        chai_1.assert.strictEqual(yGridlines.size(), yTicks.length, "There is a y gridline for each y tick");
        yGridlines.each(function (gridline, i) {
            var y = TestMethods.numAttr(d3.select(this), "y1");
            chai_1.assert.closeTo(y, yScale.scale(yTicks[i]), window.Pixel_CloseTo_Requirement, "y gridline drawn on ticks");
        });
        xScale.domain([0, 8]);
        yScale.domain([0, 8]);
        xGridlines = gridlines.content().select(".x-gridlines").selectAll("line");
        xTicks = xScale.ticks();
        xGridlines.each(function (gridline, i) {
            var x = TestMethods.numAttr(d3.select(this), "x1");
            chai_1.assert.closeTo(x, xScale.scale(xTicks[i]), window.Pixel_CloseTo_Requirement, "x gridline is updated");
        });
        yGridlines = gridlines.content().select(".y-gridlines").selectAll("line");
        yTicks = yScale.ticks();
        yGridlines.each(function (gridline, i) {
            var y = TestMethods.numAttr(d3.select(this), "y1");
            chai_1.assert.closeTo(y, yScale.scale(yTicks[i]), window.Pixel_CloseTo_Requirement, "y gridline is updated");
        });
        div.remove();
    });
    it("throws error on non-Quantitative Scales", function () {
        var categoryScale = new Plottable.Scales.Category();
        // HACKHACK #2661: Cannot assert errors being thrown with description
        chai_1.assert.throw(function () { return new Plottable.Components.Gridlines(categoryScale, null); }, Error, "xScale needs to inherit from Scale.QuantitativeScale", "can't set xScale to category scale");
        chai_1.assert.throw(function () { return new Plottable.Components.Gridlines(null, categoryScale); }, Error, "yScale needs to inherit from Scale.QuantitativeScale", "can't set yScale to category scale");
        var colorScale = new Plottable.Scales.Color();
        chai_1.assert.throw(function () { return new Plottable.Components.Gridlines(colorScale, null); }, Error, "xScale needs to inherit from Scale.QuantitativeScale", "can't set xScale to color scale");
        chai_1.assert.throw(function () { return new Plottable.Components.Gridlines(null, colorScale); }, Error, "yScale needs to inherit from Scale.QuantitativeScale", "can't set yScale to color scale");
        div.remove();
    });
});
