"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Components", function () {
    describe("PlotGroup", function () {
        var div;
        var xScale;
        var yScale;
        var plotGroup;
        var bottomPlot;
        var topPlot;
        var bottomData;
        var topData;
        beforeEach(function () {
            div = TestMethods.generateDiv();
            xScale = new Plottable.Scales.Linear();
            yScale = new Plottable.Scales.Linear();
            plotGroup = new Plottable.Components.PlotGroup();
            bottomPlot = new Plottable.Plots.Line();
            topPlot = new Plottable.Plots.Line();
            bottomData = [
                { x: 0.0, y: 0.0 },
                { x: 0.2, y: 0.2 },
            ];
            bottomPlot.x(function (d) { return d.x; }, xScale);
            bottomPlot.y(function (d) { return d.y; }, yScale);
            bottomPlot.addDataset(new Plottable.Dataset(bottomData));
            topData = [
                { x: 0.0, y: 0.0 },
                { x: 0.2, y: 0.4 },
            ];
            topPlot.x(function (d) { return d.x; }, xScale);
            topPlot.y(function (d) { return d.y; }, yScale);
            topPlot.addDataset(new Plottable.Dataset(topData));
        });
        it("throws error when appending not plot", function () {
            var label = new Plottable.Components.Label();
            chai_1.assert.throw(function () { return plotGroup.append(label); }, Error, "Plot Group only accepts plots");
            div.remove();
        });
        it("can retrieve the nearest Entity in case of empty group", function () {
            plotGroup.renderTo(div);
            chai_1.assert.isUndefined(plotGroup.entityNearest({ x: 0, y: 0 }));
            div.remove();
        });
        it("can retrieve the nearest Entity", function () {
            plotGroup.append(bottomPlot);
            plotGroup.append(topPlot);
            plotGroup.renderTo(div);
            var px = xScale.scale(topData[1].x);
            var py = yScale.scale(topData[1].y);
            var closest = plotGroup.entityNearest({ x: px, y: py });
            chai_1.assert.strictEqual(closest.datum, topData[1], "it retrieves the closest point to top plot");
            px = xScale.scale(bottomData[1].x);
            py = yScale.scale(bottomData[1].y);
            closest = plotGroup.entityNearest({ x: px, y: py });
            chai_1.assert.strictEqual(closest.datum, bottomData[1], "it retrieves the closest point to bottom plot");
            closest = plotGroup.entityNearest({ x: px, y: py + 1 });
            chai_1.assert.strictEqual(closest.datum, bottomData[1], "it retrieves the closest point in between plots");
            px = xScale.scale(topData[0].x);
            py = yScale.scale(topData[0].y);
            closest = plotGroup.entityNearest({ x: px, y: py });
            chai_1.assert.strictEqual(closest.datum, topData[0], "it retrieves the closest point to upper plot in case of tie");
            div.remove();
        });
    });
});
