"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Formatters", function () {
    describe("fixed()", function () {
        it("shows correct amount of digits according to precision", function () {
            var fixed3 = Plottable.Formatters.fixed();
            chai_1.assert.strictEqual(fixed3(1), "1.000", "shows three decimal places by defalut");
            chai_1.assert.strictEqual(fixed3(-1), "-1.000", "shows three decimal places for negative integer");
            chai_1.assert.strictEqual(fixed3(1.234), "1.234", "shows three decimal places for float");
            chai_1.assert.strictEqual(fixed3(-1.234), "-1.234", "shows three decimal placesfor for negative float");
            chai_1.assert.strictEqual(fixed3(1.2346111111), "1.235", "shows three decimal places for long float");
            chai_1.assert.strictEqual(fixed3(-1.2346111111), "-1.235", "shows three decimal places for long negatvie float");
            chai_1.assert.strictEqual(fixed3(123), "123.000", "shows three decimal places for integer");
            chai_1.assert.strictEqual(fixed3(Infinity), "Infinity", "formats non-numeric number correctly");
            chai_1.assert.strictEqual(fixed3(-Infinity), "-Infinity", "formats non-numeric number correctly");
            chai_1.assert.strictEqual(fixed3(NaN), "NaN", "formats non-numeric number correctly");
            var fixed0 = Plottable.Formatters.fixed(0);
            chai_1.assert.strictEqual(fixed0(1), "1", "shows no decimal places for integer");
            chai_1.assert.strictEqual(fixed0(-1), "-1", "shows no decimal places for negative integer");
            chai_1.assert.strictEqual(fixed0(1.23), "1", "shows no decimal places for float");
            chai_1.assert.strictEqual(fixed0(-1.23), "-1", "shows no decimal places for negative float");
            chai_1.assert.strictEqual(fixed0(123.456), "123", "shows no decimal places for integer");
        });
        it.skip("throws exception for non-numeric values", function () {
            var nonNumericValues = [null, undefined, "123", "abc", ""];
            var fixed = Plottable.Formatters.fixed();
            nonNumericValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return fixed(value); }, Error, "error message TBD", value + " is not a valid value for Formatter.fixed");
            });
        });
        it("throws exception for invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.fixed(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.fixed(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
    });
    describe("general()", function () {
        it("shows correct amount of digits according to precision", function () {
            var general3 = Plottable.Formatters.general();
            chai_1.assert.strictEqual(general3(1), "1", "does not pad 0 to three decimal places");
            chai_1.assert.strictEqual(general3(-1), "-1", "does not pad 0 to three decimal places");
            chai_1.assert.strictEqual(general3(1.234), "1.234", "shows up to three decimal places");
            chai_1.assert.strictEqual(general3(-1.234), "-1.234", "shows up to three decimal places");
            chai_1.assert.strictEqual(general3(1.2346), "1.235", "shows up to three decimal places");
            chai_1.assert.strictEqual(general3(-1.2346), "-1.235", "shows up to three decimal places");
            var general2 = Plottable.Formatters.general(2);
            chai_1.assert.strictEqual(general2(1), "1", "does not pad 0 to two decimal places");
            chai_1.assert.strictEqual(general2(-1), "-1", "does not pad 0 to two decimal places");
            chai_1.assert.strictEqual(general2(1.2), "1.2", "does not pad 0 to two decimal places");
            chai_1.assert.strictEqual(general2(-1.2), "-1.2", "does not pad 0 to two decimal places");
            chai_1.assert.strictEqual(general2(1.23), "1.23", "shows up to two decimal places");
            chai_1.assert.strictEqual(general2(-1.23), "-1.23", "shows up to two decimal places");
            chai_1.assert.strictEqual(general2(1.235), "1.24", "shows up to two decimal places");
            chai_1.assert.strictEqual(general2(-1.235), "-1.24", "shows up to two decimal places");
            var general0 = Plottable.Formatters.general(0);
            chai_1.assert.strictEqual(general0(1), "1", "shows no decimals");
            chai_1.assert.strictEqual(general0(1.535), "2", "shows no decimals");
        });
        it("stringifies non-numeric values", function () {
            var general = Plottable.Formatters.general();
            var nonNumericValues = [null, undefined, Infinity, -Infinity, NaN, "123", "abc"];
            var stringifiedValues = ["null", "undefined", "Infinity", "-Infinity", "NaN", "123", "abc"];
            nonNumericValues.forEach(function (value, i) {
                return chai_1.assert.strictEqual(general(value), stringifiedValues[i], "non-numeric input " + value + " is stringified");
            });
        });
        it("throws an error on invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.general(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.general(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
    });
    describe("identity()", function () {
        it("stringifies inputs", function () {
            var identity = Plottable.Formatters.identity();
            var values = [1, 0.999999, null, undefined, Infinity, -Infinity, NaN, "123", "abc", ""];
            var stringifiedValues = ["1", "0.999999", "null", "undefined", "Infinity", "-Infinity", "NaN", "123", "abc", ""];
            values.forEach(function (value, i) {
                return chai_1.assert.strictEqual(identity(value), stringifiedValues[i], value + " is stringified");
            });
        });
    });
    describe("currency()", function () {
        it("formats values based on precision, symobl, and prefix correctly", function () {
            var defaultFormatter = Plottable.Formatters.currency();
            chai_1.assert.strictEqual(defaultFormatter(1), "$1.00", "formatted correctly with default \"$\" prefix");
            chai_1.assert.strictEqual(defaultFormatter(-1), "-$1.00", "formatted negative integer correctly with default \"$\" prefix");
            chai_1.assert.strictEqual(defaultFormatter(1.999), "$2.00", "formatted correctly with default \"$\" prefix");
            chai_1.assert.strictEqual(defaultFormatter(-1.234), "-$1.23", "formatted negative float correctly with default \"$\" prefix");
            var currencyFormatter0 = Plottable.Formatters.currency(0);
            chai_1.assert.strictEqual(currencyFormatter0(1.1234), "$1", "formatted with correct precision");
            chai_1.assert.strictEqual(currencyFormatter0(123), "$123", "formatted with correct precision");
            var poundFormatter = Plottable.Formatters.currency(2, "£");
            chai_1.assert.strictEqual(poundFormatter(1.1234), "£1.12", "formatted with correct precision and \"£\" as prefix");
            chai_1.assert.strictEqual(poundFormatter(-100.987), "-£100.99", "formatted with correct precision and \"£\" as prefix");
            var centsFormatter = Plottable.Formatters.currency(0, "c", false);
            chai_1.assert.strictEqual(centsFormatter(1), "1c", "formatted correctly with \"c\" as suffix");
            chai_1.assert.strictEqual(centsFormatter(-1), "-1c", "formatted correctly with \"c\" as suffix");
        });
        it("throws an error on invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.currency(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.currency(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
        it.skip("throws exception for non-numeric values", function () {
            var nonNumericValues = [null, undefined, "123", "abc", ""];
            // "$0.00", "$NaN", "$123.00", "$NaN", "$0.00"
            var defaultFormatter = Plottable.Formatters.currency();
            nonNumericValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return defaultFormatter(value); }, Error, "error message TBD", value + " is not a valid value for Formatter.currency");
            });
        });
    });
    describe("multiTime()", function () {
        it("uses reasonable defaults", function () {
            var timeFormatter = Plottable.Formatters.multiTime();
            // year, month, day, hours, minutes, seconds, milliseconds
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 0, 1, 0, 0, 0, 0)), "2000", "only the year was displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 1, 0, 0, 0, 0)), "Mar", "only the month was displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 5, 0, 0, 0, 0)), "Mar 05", "month and date displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 2, 0, 0, 0, 0)), "Thu 02", "day and date displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 1, 20, 0, 0, 0)), "08 PM", "only hour was displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 1, 20, 34, 0, 0)), "08:34", "hour and minute was displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 2, 1, 20, 34, 53, 0)), ":53", "seconds was displayed");
            chai_1.assert.strictEqual(timeFormatter(new Date(2000, 0, 1, 0, 0, 0, 950)), ".950", "milliseconds was displayed");
        });
        it.skip("throws exception for non-Date values", function () {
            var nonNumericValues = [null, undefined, NaN, "123", "abc", "", 0];
            var timeFormatter = Plottable.Formatters.multiTime();
            nonNumericValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return timeFormatter(value); }, Error, "error message TBD", value + " is not a valid value for Formatter.multiTime");
            });
        });
    });
    describe("percentage()", function () {
        it("formats number to percetage with correct precision", function () {
            var percentFormatter = Plottable.Formatters.percentage();
            chai_1.assert.strictEqual(percentFormatter(1), "100%", "the value was multiplied by 100, a percent sign was appended, and no decimal places are shown by default");
            chai_1.assert.strictEqual(percentFormatter(0.07), "7%", "does not have trailing zeros and is not empty string");
            chai_1.assert.strictEqual(percentFormatter(-0.355), "-36%", "negative values are formatted correctly");
            chai_1.assert.strictEqual(percentFormatter(50), "5000%", "formats 2 digit integer correctly");
            var twoPercentFormatter = Plottable.Formatters.percentage(2);
            chai_1.assert.strictEqual(twoPercentFormatter(0.0035), "0.35%", "works even if multiplying by 100 does not make it an integer");
            chai_1.assert.strictEqual(twoPercentFormatter(-0.123), "-12.30%", "add 0 padding to make precision");
        });
        it.skip("formats non-numeric number correctly", function () {
            var percentFormatter = Plottable.Formatters.percentage();
            chai_1.assert.strictEqual(percentFormatter(NaN), "NaN", "stringifies non-numeric number");
            chai_1.assert.strictEqual(percentFormatter(Infinity), "Infinity", "stringifies non-numeric number");
            chai_1.assert.strictEqual(percentFormatter(-Infinity), "-Infinity", "stringifies non-numeric number");
        });
        it("throws an error on invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.percentage(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.percentage(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
        it.skip("throws exception for non-numeric values", function () {
            var nonNumericValues = [null, undefined, "123", "abc", ""];
            var percentFormatter = Plottable.Formatters.percentage();
            nonNumericValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return percentFormatter(value); }, Error, "error message TBD", value + " is not a valid value for Formatter.percentage");
            });
        });
    });
    describe("siSuffix()", function () {
        it("shortens long numbers", function () {
            var lnFormatter = Plottable.Formatters.siSuffix();
            chai_1.assert.strictEqual(lnFormatter(1), "1.00", "shows 3 signifigicant figures by default");
            chai_1.assert.operator(lnFormatter(Math.pow(10, 12)).length, "<=", 5, "large number was formatted to a short string");
            chai_1.assert.operator(lnFormatter(Math.pow(10, -12)).length, "<=", 5, "small number was formatted to a short string");
            chai_1.assert.strictEqual(lnFormatter(Math.pow(10, 5) * 1.55555), "156k", "formatting respects precision");
            chai_1.assert.strictEqual(lnFormatter(-Math.pow(10, 6) * 23.456), "-23.5M", "formatting respects precision");
            var lnFormatter2 = Plottable.Formatters.siSuffix(2);
            chai_1.assert.strictEqual(lnFormatter2(1), "1.0", "shows 2 signifigicant figures by default");
            chai_1.assert.strictEqual(lnFormatter2(Math.pow(10, 5) * 1.55555), "160k", "formatting respects precision");
            chai_1.assert.strictEqual(lnFormatter2(-Math.pow(10, 6) * 23.456), "-23M", "formatting respects precision");
        });
        it.skip("formats non-numeric number correctly", function () {
            var lnFormatter = Plottable.Formatters.siSuffix();
            chai_1.assert.strictEqual(lnFormatter(NaN), "NaN", "stringifies non-numeric number");
            chai_1.assert.strictEqual(lnFormatter(Infinity), "Infinity", "stringifies non-numeric number");
            chai_1.assert.strictEqual(lnFormatter(-Infinity), "-Infinity", "stringifies non-numeric number");
        });
        it("throws an error on invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.siSuffix(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.siSuffix(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
        it.skip("throws exception for non-numeric values", function () {
            var nonNumericValues = [null, undefined, "123", "abc", ""];
            var lnFormatter = Plottable.Formatters.siSuffix();
            nonNumericValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return lnFormatter(value); }, Error, "error message TBD", value + " is not a valid value for Formatter.siSuffix");
            });
        });
    });
    describe("shortScale()", function () {
        it("shortens long numbers", function () {
            var formatter = Plottable.Formatters.shortScale();
            chai_1.assert.strictEqual(formatter(1), "1.000", "small numbers format simply");
            chai_1.assert.strictEqual(formatter(-1), "-1.000", "negative small numbers format simply");
            chai_1.assert.strictEqual(formatter(2e3), "2.000K", "thousands numbers format using short scale magnitudes");
            chai_1.assert.strictEqual(formatter(2e6), "2.000M", "millions numbers format using short scale magnitudes");
            chai_1.assert.strictEqual(formatter(2e9), "2.000B", "billions numbers format using short scale magnitudes");
            chai_1.assert.strictEqual(formatter(2e12), "2.000T", "trillions magnitude numbers format using short scale magnitudes");
            chai_1.assert.strictEqual(formatter(2e15), "2.000Q", "quadrillions numbers format using short scale magnitudes");
            chai_1.assert.strictEqual(formatter(999.999), "999.999", "boundary cases are correct");
            chai_1.assert.strictEqual(formatter(999.99999), "1.000K", "rounding cases are correct");
            chai_1.assert.strictEqual(formatter(-999.99999), "-1.000K", "negative rounding cases are correct");
            chai_1.assert.strictEqual(formatter(1000000), "1.000M", "boundary cases are correct");
            chai_1.assert.strictEqual(formatter(999999000000000000), "999.999Q", "Largest positive short-scale representable number");
            chai_1.assert.strictEqual(formatter(999999400000000000), "999.999Q", "Largest positive short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter(999999500000000000), "1.000e+18", "Largest positive short-scale non-representable number round-up");
            chai_1.assert.strictEqual(formatter(1e17), "100.000Q", "Last positive round short-scale representable number");
            chai_1.assert.strictEqual(formatter(1e18), "1.000e+18", "First positive round short-scale non-representable number");
            chai_1.assert.strictEqual(formatter(-999999000000000000), "-999.999Q", "Largest negative short-scale representable number");
            chai_1.assert.strictEqual(formatter(999999400000000000), "999.999Q", "Largest negative short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter(999999500000000000), "1.000e+18", "Largest negative short-scale non-representable number round-up");
            chai_1.assert.strictEqual(formatter(-1e17), "-100.000Q", "Last negative round short-scale representable number");
            chai_1.assert.strictEqual(formatter(-1e18), "-1.000e+18", "First negative round short-scale non-representable number");
            chai_1.assert.strictEqual(formatter(0.001), "0.001", "Smallest positive short-scale representable number");
            chai_1.assert.strictEqual(formatter(0.0009), "9.000e-4", "Round up is not applied for very small positive numbers");
            chai_1.assert.strictEqual(formatter(-0.001), "-0.001", "Smallest negative short-scale representable number");
            chai_1.assert.strictEqual(formatter(-0.0009), "-9.000e-4", "Round up is not applied for very small negative numbers");
            chai_1.assert.strictEqual(formatter(0), "0.000", "0 gets formatted well");
            chai_1.assert.strictEqual(formatter(-0), "0.000", "-0 gets formatted well");
            chai_1.assert.strictEqual(formatter(Infinity), "Infinity", "Infinity edge case");
            // this is actually failing because of d3 - see https://github.com/d3/d3-format/issues/42
            // assert.strictEqual(formatter(-Infinity), "-Infinity", "-Infinity edge case");
            chai_1.assert.strictEqual(formatter(NaN), "NaN", "NaN edge case");
            chai_1.assert.strictEqual(formatter(1e37), "1.000e+37", "large magnitute number use scientific notation");
            chai_1.assert.strictEqual(formatter(1e-7), "1.000e-7", "small magnitude numbers use scientific notation");
        });
        it("respects the precision provided", function () {
            var formatter = Plottable.Formatters.shortScale(1);
            chai_1.assert.strictEqual(formatter(1), "1.0", "Just one decimal digit is added");
            chai_1.assert.strictEqual(formatter(999), "999.0", "Conversion to K happens in the same place (lower)");
            chai_1.assert.strictEqual(formatter(1000), "1.0K", "Conversion to K happens in the same place (upper)");
            chai_1.assert.strictEqual(formatter(999900000000000000), "999.9Q", "Largest positive short-scale representable number");
            chai_1.assert.strictEqual(formatter(999940000000000000), "999.9Q", "Largest positive short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter(999950000000000000), "1.0e+18", "Largest positive short-scale representable number round-up");
            chai_1.assert.strictEqual(formatter(-999900000000000000), "-999.9Q", "Largest negative short-scale representable number");
            chai_1.assert.strictEqual(formatter(-999940000000000000), "-999.9Q", "Largest negative short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter(-999950000000000000), "-1.0e+18", "Largest negative short-scale representable number round-up");
            chai_1.assert.strictEqual(formatter(0.1), "0.1", "Smallest positive short-scale representable number");
            chai_1.assert.strictEqual(formatter(0.09), "9.0e-2", "Round up is not applied for very small positive numbers");
            chai_1.assert.strictEqual(formatter(-0.1), "-0.1", "Smallest negative short-scale representable number");
            chai_1.assert.strictEqual(formatter(-0.09), "-9.0e-2", "Round up is not applied for very small negative numbers");
            chai_1.assert.strictEqual(formatter(0), "0.0", "0 gets formatted well");
            var formatter0 = Plottable.Formatters.shortScale(0);
            chai_1.assert.strictEqual(formatter0(1), "1", "Just one decimal digit is added");
            chai_1.assert.strictEqual(formatter0(999), "999", "Conversion to K happens in the same place (lower)");
            chai_1.assert.strictEqual(formatter0(1000), "1K", "Conversion to K happens in the same place (upper)");
            chai_1.assert.strictEqual(formatter0(999000000000000000), "999Q", "Largest positive short-scale representable number");
            chai_1.assert.strictEqual(formatter0(999400000000000000), "999Q", "Largest positive short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter0(999500000000000000), "1e+18", "Largest positive short-scale representable number round-up");
            chai_1.assert.strictEqual(formatter0(-999000000000000000), "-999Q", "Largest negative short-scale representable number");
            chai_1.assert.strictEqual(formatter0(-999400000000000000), "-999Q", "Largest negative short-scale representable number round-down");
            chai_1.assert.strictEqual(formatter0(-999500000000000000), "-1e+18", "Largest negative short-scale representable number round-up");
            chai_1.assert.strictEqual(formatter0(0.9), "9e-1", "Round up is not applied for very small positive numbers");
            chai_1.assert.strictEqual(formatter0(0), "0", "0 gets formatted well");
        });
        it("throws an error on invalid precision", function () {
            var nonIntegerValues = [null, 2.1, NaN, "5", "abc"];
            nonIntegerValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.shortScale(value); }, Error, "Formatter precision must be an integer", value + " is not a valid precision value");
            });
            var outOfBoundValues = [-1, 21, -Infinity, Infinity];
            outOfBoundValues.forEach(function (value) {
                return chai_1.assert.throws(function () { return Plottable.Formatters.shortScale(value); }, Error, "Formatter precision must be between 0 and 20", value + " is not a valid precision value");
            });
        });
    });
});
