"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("RenderController", function () {
    var DIV_WIDTH = 400;
    var DIV_HEIGHT = 300;
    describe("configuring the render policy", function () {
        after(function () {
            Plottable.RenderController.renderPolicy(Plottable.RenderController.Policy.immediate);
        });
        it("can set a render policy", function () {
            Plottable.RenderController.renderPolicy(Plottable.RenderController.Policy.timeout);
            chai_1.assert.strictEqual(Object.getPrototypeOf(Plottable.RenderController.renderPolicy()), Plottable.RenderPolicies.Timeout.prototype, "render policy is of the same type");
        });
        it("throws a warning for unrecognized render policies", function () {
            var unrecognizedRenderPolicy = "foo";
            TestMethods.assertWarns(function () { return Plottable.RenderController.renderPolicy(unrecognizedRenderPolicy); }, "Unrecognized renderPolicy", "warning sent for unrecognized render policy");
        });
    });
    it("can queue a component to render", function () {
        var div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        var component = new Plottable.Component();
        var renderedClass = "rendered";
        component.renderImmediately = function () {
            component.content().append("g").classed(renderedClass, true);
            return component;
        };
        component.anchor(div);
        Plottable.RenderController.registerToRender(component);
        chai_1.assert.isFalse(component.content().select("." + renderedClass).empty(), "component has rendered");
        component.destroy();
        div.remove();
    });
    it("can queue a component to undergo layout computation and render", function () {
        var div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
        var component = new Plottable.Component();
        var renderedClass = "rendered";
        component.renderImmediately = function () {
            component.content().append("g").classed(renderedClass, true);
            return component;
        };
        component.anchor(div);
        Plottable.RenderController.registerToComputeLayoutAndRender(component);
        chai_1.assert.isFalse(component.content().select("." + renderedClass).empty(), "component has rendered");
        chai_1.assert.deepEqual(component.origin(), { x: 0, y: 0 }, "origin set");
        chai_1.assert.strictEqual(component.width(), DIV_WIDTH, "width set");
        chai_1.assert.strictEqual(component.height(), DIV_HEIGHT, "height set");
        component.destroy();
        div.remove();
    });
    // HACKHACK: https://github.com/palantir/plottable/issues/2083
    it.skip("can render components that are triggered by another component's render", function () {
        var link1 = new Plottable.Component();
        var div1 = TestMethods.generateDiv();
        link1.anchor(div1).computeLayout();
        var link2 = new Plottable.Component();
        var div2 = TestMethods.generateDiv();
        link2.anchor(div2).computeLayout();
        link1.renderImmediately = function () { return link2.render(); };
        var link2Rendered = false;
        link2.renderImmediately = function () { return link2Rendered = true; };
        link1.render();
        chai_1.assert.isTrue(link2Rendered, "dependent Component was render()-ed");
        div1.remove();
        div2.remove();
    });
});
