"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Scales", function () {
    describe("Category Scale", function () {
        describe("Basic usage", function () {
            var scale;
            beforeEach(function () {
                scale = new Plottable.Scales.Category();
            });
            it("updates rangeBand when domain changes", function () {
                scale.range([0, 2679]);
                scale.domain(["1", "2", "3", "4"]);
                chai_1.assert.closeTo(scale.rangeBand(), 399, 1);
                scale.domain(["1", "2", "3", "4", "5"]);
                chai_1.assert.closeTo(scale.rangeBand(), 329, 1);
            });
            it("computes the correct stepWidth", function () {
                scale.range([0, 3000]);
                scale.domain(["1", "2", "3", "4"]);
                var widthSum = scale.rangeBand() * (1 + scale.innerPadding());
                chai_1.assert.closeTo(scale.stepWidth(), widthSum, 1e-5, "step width is the sum of innerPadding width and band width");
            });
            it("interacts well with BarPlot and data swapping", function () {
                // This unit test taken from SLATE, see SLATE-163 a fix for SLATE-102
                var xScale = new Plottable.Scales.Category();
                var yScale = new Plottable.Scales.Linear();
                var dA = { x: "A", y: 2 };
                var dB = { x: "B", y: 2 };
                var dC = { x: "C", y: 2 };
                var dataset = new Plottable.Dataset([dA, dB]);
                var barPlot = new Plottable.Plots.Bar();
                barPlot.addDataset(dataset);
                barPlot.x(function (d) { return d.x; }, xScale);
                barPlot.y(function (d) { return d.y; }, yScale);
                var div = TestMethods.generateDiv();
                chai_1.assert.deepEqual(xScale.domain(), [], "before anchoring, the bar plot doesn't proxy data to the scale");
                barPlot.renderTo(div);
                chai_1.assert.deepEqual(xScale.domain(), ["A", "B"], "after anchoring, the bar plot's data is on the scale");
                iterateDataChanges([], [dA, dB, dC], []);
                chai_1.assert.lengthOf(xScale.domain(), 0);
                iterateDataChanges([dA], [dB]);
                chai_1.assert.lengthOf(xScale.domain(), 1);
                iterateDataChanges([], [dA, dB, dC]);
                chai_1.assert.lengthOf(xScale.domain(), 3);
                div.remove();
                function iterateDataChanges() {
                    var dataChanges = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        dataChanges[_i] = arguments[_i];
                    }
                    dataChanges.forEach(function (dataChange) {
                        dataset.data(dataChange);
                    });
                }
            });
        });
        describe("Extent calculation", function () {
            var scale;
            beforeEach(function () {
                scale = new Plottable.Scales.Category();
            });
            it("gives the unique values when domain is stringy", function () {
                var values = ["1", "3", "2", "1"];
                var computedExtent = scale.extentOfValues(values);
                chai_1.assert.deepEqual(computedExtent, ["1", "3", "2"], "the extent is made of all the unique values in the domain");
            });
            it("gives the unique values even when domain is numeric", function () {
                var values = [1, 3, 2, 1];
                var computedExtent = scale.extentOfValues(values);
                chai_1.assert.deepEqual(computedExtent, [1, 3, 2], "the extent is made of all the unique values in the domain");
            });
        });
    });
});
