"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Scales", function () {
    describe("Interploated Color Scale", function () {
        describe("Basic usage", function () {
            var scale;
            beforeEach(function () {
                scale = new Plottable.Scales.InterpolatedColor();
            });
            it("defaults to a linear scale and a red color palette", function () {
                scale.domain([0, 16]);
                chai_1.assert.deepEqual(d3.color(scale.scale(0)), d3.color("#ffffff"), "domain minimum maps to white");
                chai_1.assert.deepEqual(d3.color(scale.scale(8)), d3.color("#feb24c"), "domain median maps in between red and white");
                chai_1.assert.deepEqual(d3.color(scale.scale(16)), d3.color("#b10026"), "domain maximum maps to red");
                chai_1.assert.deepEqual(scale.range(), Plottable.Scales.InterpolatedColor.REDS, "the range of the default scale is made of shades of red");
                scale.domain([0, 1]);
                chai_1.assert.deepEqual(d3.color(scale.scale(1)), d3.color("#b10026"), "new domain maximum maps to red");
                chai_1.assert.deepEqual(d3.color(scale.scale(0.5)), d3.color("#feb24c"), "new domain median maps in between red and white");
                chai_1.assert.deepEqual(d3.color(scale.scale(0.9)), d3.color("#d9151f"), "different shades of red are obtained for different values");
            });
            it("accepts array types with color hex values", function () {
                scale.range(["#000", "#FFF"]);
                scale.domain([0, 16]);
                chai_1.assert.deepEqual(d3.color(scale.scale(0)), d3.color("#000000"), "domain minimum maps to black");
                chai_1.assert.deepEqual(d3.color(scale.scale(8)), d3.color("#777777"), "domain median maps to gray");
                chai_1.assert.deepEqual(d3.color(scale.scale(16)), d3.color("#ffffff"), "domain maximum maps to white");
            });
            it("accepts array types with color names", function () {
                scale.range(["white", "black"]);
                scale.domain([0, 16]);
                chai_1.assert.deepEqual(d3.color(scale.scale(0)), d3.color("#ffffff"), "domain minimum maps to white");
                chai_1.assert.deepEqual(d3.color(scale.scale(8)), d3.color("#777777"), "domain median maps to gray");
                chai_1.assert.deepEqual(d3.color(scale.scale(16)), d3.color("#000000"), "domain maximum maps to black");
            });
            it("clamps overflow and underflow values to range", function () {
                scale.range(["black", "white"]);
                scale.domain([0, 16]);
                chai_1.assert.deepEqual(d3.color(scale.scale(0)), d3.color("#000000"), "domain minimum maps to black");
                chai_1.assert.deepEqual(d3.color(scale.scale(16)), d3.color("#ffffff"), "domain maximum maps to white");
                chai_1.assert.deepEqual(d3.color(scale.scale(-100)), d3.color("#000000"), "values smaller than the domain minimum clamp to black");
                chai_1.assert.deepEqual(d3.color(scale.scale(100)), d3.color("#ffffff"), "values larger than the domain maximum clamp to white");
            });
        });
    });
});
