"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Utils.DOM", function () {
    it("getBBox works properly", function () {
        var svg = TestMethods.generateSVG();
        var expectedBox = {
            x: 0,
            y: 0,
            width: 40,
            height: 20,
        };
        var rect = svg.append("rect").attrs(expectedBox);
        var measuredBox = Plottable.Utils.DOM.elementBBox(rect);
        chai_1.assert.deepEqual(measuredBox, expectedBox, "getBBox measures correctly");
        svg.remove();
    });
    it("getBBox does not fail on disconnected and display:none nodes", function () {
        var expectedBox = {
            x: 0,
            y: 0,
            width: 40,
            height: 20,
        };
        var removedSVG = TestMethods.generateSVG().remove();
        var rect = removedSVG.append("rect").attrs(expectedBox);
        Plottable.Utils.DOM.elementBBox(rect); // could throw NS_ERROR on FF
        var noneSVG = TestMethods.generateSVG().style("display", "none");
        rect = noneSVG.append("rect").attrs(expectedBox);
        Plottable.Utils.DOM.elementBBox(rect); // could throw NS_ERROR on FF
        noneSVG.remove();
    });
    describe("elementWidth(), elementHeight()", function () {
        it("can get a plain element's size", function () {
            var parent = TestMethods.getElementParent();
            parent.style("width", "300px");
            parent.style("height", "200px");
            var parentElem = parent.node();
            var width = Plottable.Utils.DOM.elementWidth(parentElem);
            chai_1.assert.strictEqual(width, 300, "measured width matches set width");
            var height = Plottable.Utils.DOM.elementHeight(parentElem);
            chai_1.assert.strictEqual(height, 200, "measured height matches set height");
        });
        it("can get the svg's size", function () {
            var svg = TestMethods.generateSVG(450, 120);
            var svgElem = svg.node();
            var width = Plottable.Utils.DOM.elementWidth(svgElem);
            chai_1.assert.strictEqual(width, 450, "measured width matches set width");
            var height = Plottable.Utils.DOM.elementHeight(svgElem);
            chai_1.assert.strictEqual(height, 120, "measured height matches set height");
            svg.remove();
        });
        it("can accept multiple units and convert to pixels", function () {
            var parent = TestMethods.getElementParent();
            var parentElem = parent.node();
            var child = parent.append("div");
            var childElem = child.node();
            parent.style("width", "200px");
            parent.style("height", "50px");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementWidth(parentElem), 200, "width is correct");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementHeight(parentElem), 50, "height is correct");
            child.style("width", "20px");
            child.style("height", "10px");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementWidth(childElem), 20, "width is correct");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementHeight(childElem), 10, "height is correct");
            child.style("width", "100%");
            child.style("height", "100%");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementWidth(childElem), 200, "width is correct");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementHeight(childElem), 50, "height is correct");
            child.style("width", "50%");
            child.style("height", "50%");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementWidth(childElem), 100, "width is correct");
            chai_1.assert.strictEqual(Plottable.Utils.DOM.elementHeight(childElem), 25, "height is correct");
            // reset test page DOM
            parent.style("width", "auto");
            parent.style("height", "auto");
            child.remove();
        });
    });
});
