"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Utils.Window", function () {
    describe("warn()", function () {
        if (console != null) {
            it("uses console.warn if available, console.log otherwise", function () {
                var originalWindowWarn = console.warn;
                var receivedWarning = null;
                var replacementWarn = function (warning) { return receivedWarning = warning; };
                console.warn = replacementWarn;
                var expectedWarning = "BEEP BOOP";
                Plottable.Utils.Window.warn(expectedWarning);
                console.warn = originalWindowWarn;
                chai_1.assert.strictEqual(receivedWarning, expectedWarning, "console.warn was passed the expected message");
                var originalWindowLog = console.log;
                var receivedMessage = null;
                var replacementLog = function (warning) { return receivedMessage = warning; };
                console.warn = null;
                console.log = replacementLog;
                receivedWarning = null;
                Plottable.Utils.Window.warn(expectedWarning);
                console.warn = originalWindowWarn;
                console.log = originalWindowLog;
                chai_1.assert.isNull(receivedWarning, "console.warn was not invoked if it doesn't exist");
                chai_1.assert.strictEqual(receivedMessage, expectedWarning, "used console.log as a fallback");
            });
            it("does not emit warnings if Configs.SHOW_WARNINGS is set to false", function () {
                var originalWindowWarn = console.warn;
                var windowWarnWasCalled = false;
                var replacementWarn = function (warning) { return windowWarnWasCalled = true; };
                console.warn = replacementWarn;
                var originalShowWarnings = Plottable.Configs.SHOW_WARNINGS;
                Plottable.Configs.SHOW_WARNINGS = false;
                Plottable.Utils.Window.warn("BEEP BOOP");
                console.warn = originalWindowWarn;
                Plottable.Configs.SHOW_WARNINGS = originalShowWarnings;
                chai_1.assert.isFalse(windowWarnWasCalled, "no warning was logged if SHOW_WARNINGS was set to false");
            });
        }
    });
});
